@@ -18,25 +18,24 @@ use crypto::digest::Digest;
 use crypto::sha2::Sha512;
 use sabre_sdk::protocol::state::{
     ContractBuilder, ContractRegistry, ContractRegistryBuilder, NamespaceRegistry,
-    NamespaceRegistryBuilder, PermissionBuilder, SmartPermissionBuilder, VersionBuilder,
+    NamespaceRegistryBuilder, PermissionBuilder, VersionBuilder,
 };
 use sawtooth_sdk::messages::processor::TpProcessRequest;
 use sawtooth_sdk::processor::handler::ApplyError;
 use sawtooth_sdk::processor::handler::TransactionContext;
 use sawtooth_sdk::processor::handler::TransactionHandler;
 
+use crate::admin::AdminPermission;
 use crate::payload::SabreRequestPayload;
 use crate::state::SabreState;
 use crate::wasm_executor::wasm_module::WasmModule;
 use sabre_sdk::protocol::payload::{
     Action, CreateContractAction, CreateContractRegistryAction, CreateNamespaceRegistryAction,
-    CreateNamespaceRegistryPermissionAction, CreateSmartPermissionAction, DeleteContractAction,
-    DeleteContractRegistryAction, DeleteNamespaceRegistryAction,
-    DeleteNamespaceRegistryPermissionAction, DeleteSmartPermissionAction, ExecuteContractAction,
+    CreateNamespaceRegistryPermissionAction, DeleteContractAction, DeleteContractRegistryAction,
+    DeleteNamespaceRegistryAction, DeleteNamespaceRegistryPermissionAction, ExecuteContractAction,
     UpdateContractRegistryOwnersAction, UpdateNamespaceRegistryOwnersAction,
-    UpdateSmartPermissionAction,
 };
-use sabre_sdk::protocol::{ADMINISTRATORS_SETTING_KEY, SABRE_PROTOCOL_VERSION};
+use sabre_sdk::protocol::SABRE_PROTOCOL_VERSION;
 
 /// The namespace registry prefix for global state (00ec00)
 const NAMESPACE_REGISTRY_PREFIX: &str = "00ec00";
@@ -60,12 +59,12 @@ pub struct SabreTransactionHandler {
     family_name: String,
     family_versions: Vec<String>,
     namespaces: Vec<String>,
+    admin_permissions: Box<dyn AdminPermission>,
 }
 
 impl SabreTransactionHandler {
     /// Constructs a new SabreTransactionHandler
-    #[allow(clippy::new_without_default)]
-    pub fn new() -> SabreTransactionHandler {
+    pub fn new(admin_permissions: Box<dyn AdminPermission>) -> SabreTransactionHandler {
         SabreTransactionHandler {
             family_name: "sabre".into(),
             family_versions: vec!["0.5".into(), "0.6".into(), SABRE_PROTOCOL_VERSION.into()],
@@ -74,6 +73,7 @@ impl SabreTransactionHandler {
                 CONTRACT_REGISTRY_PREFIX.into(),
                 CONTRACT_PREFIX.into(),
             ],
+            admin_permissions,
         }
     }
 }
@@ -135,29 +135,51 @@ impl TransactionHandler for SabreTransactionHandler {
                 &mut state,
             ),
             Action::CreateContractRegistry(create_contract_registry_payload) => {
-                create_contract_registry(create_contract_registry_payload, signer, &mut state)
+                create_contract_registry(
+                    create_contract_registry_payload,
+                    signer,
+                    &mut state,
+                    &*self.admin_permissions,
+                )
             }
             Action::DeleteContractRegistry(delete_contract_registry_payload) => {
-                delete_contract_registry(delete_contract_registry_payload, signer, &mut state)
+                delete_contract_registry(
+                    delete_contract_registry_payload,
+                    signer,
+                    &mut state,
+                    &*self.admin_permissions,
+                )
             }
             Action::UpdateContractRegistryOwners(update_contract_registry_owners_payload) => {
                 update_contract_registry_owners(
                     update_contract_registry_owners_payload,
                     signer,
                     &mut state,
+                    &*self.admin_permissions,
                 )
             }
             Action::CreateNamespaceRegistry(create_namespace_registry_payload) => {
-                create_namespace_registry(create_namespace_registry_payload, signer, &mut state)
+                create_namespace_registry(
+                    create_namespace_registry_payload,
+                    signer,
+                    &mut state,
+                    &*self.admin_permissions,
+                )
             }
             Action::DeleteNamespaceRegistry(delete_namespace_registry_payload) => {
-                delete_namespace_registry(delete_namespace_registry_payload, signer, &mut state)
+                delete_namespace_registry(
+                    delete_namespace_registry_payload,
+                    signer,
+                    &mut state,
+                    &*self.admin_permissions,
+                )
             }
             Action::UpdateNamespaceRegistryOwners(update_namespace_registry_owners_payload) => {
                 update_namespace_registry_owners(
                     update_namespace_registry_owners_payload,
                     signer,
                     &mut state,
+                    &*self.admin_permissions,
                 )
             }
             Action::CreateNamespaceRegistryPermission(
@@ -166,6 +188,7 @@ impl TransactionHandler for SabreTransactionHandler {
                 create_namespace_registry_permission_payload,
                 signer,
                 &mut state,
+                &*self.admin_permissions,
             ),
             Action::DeleteNamespaceRegistryPermission(
                 delete_namespace_registry_permission_payload,
@@ -173,16 +196,8 @@ impl TransactionHandler for SabreTransactionHandler {
                 delete_namespace_registry_permission_payload,
                 signer,
                 &mut state,
+                &*self.admin_permissions,
             ),
-            Action::CreateSmartPermission(payload) => {
-                create_smart_permission(payload, signer, &mut state)
-            }
-            Action::UpdateSmartPermission(payload) => {
-                update_smart_permission(payload, signer, &mut state)
-            }
-            Action::DeleteSmartPermission(payload) => {
-                delete_smart_permission(payload, signer, &mut state)
-            }
         }
     }
 }
@@ -507,6 +522,7 @@ fn create_contract_registry(
     payload: CreateContractRegistryAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let name = payload.name();
 
@@ -526,33 +542,11 @@ fn create_contract_registry(
         }
     };
 
-    let setting = match state.get_admin_setting() {
-        Ok(Some(setting)) => setting,
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Admins not set and only admins can create a contract registry: {}",
-                signer,
-            )));
-        }
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Unable to check state: {}",
-                err,
-            )));
-        }
-    };
-
-    for entry in setting.get_entries() {
-        if entry.key == ADMINISTRATORS_SETTING_KEY {
-            let values = entry.value.split(',');
-            let value_vec: Vec<&str> = values.collect();
-            if !value_vec.contains(&signer) {
-                return Err(ApplyError::InvalidTransaction(format!(
-                    "Only admins can create a contract registry: {}",
-                    signer,
-                )));
-            }
-        }
+    if !admin_permissions.is_admin(signer, state)? {
+        return Err(ApplyError::InvalidTransaction(format!(
+            "Only admins can create a contract registry: {}",
+            signer,
+        )));
     }
 
     let contract_registry = ContractRegistryBuilder::new()
@@ -570,6 +564,7 @@ fn delete_contract_registry(
     payload: DeleteContractRegistryAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let name = payload.name();
     let contract_registry = match state.get_contract_registry(name) {
@@ -596,7 +591,7 @@ fn delete_contract_registry(
     }
 
     // Check if signer is an owner or an admin
-    can_update_contract_registry(contract_registry, signer, state)?;
+    can_update_contract_registry(contract_registry, signer, state, admin_permissions)?;
 
     state.delete_contract_registry(name)
 }
@@ -605,6 +600,7 @@ fn update_contract_registry_owners(
     payload: UpdateContractRegistryOwnersAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let name = payload.name();
     let contract_registry = match state.get_contract_registry(name) {
@@ -624,7 +620,7 @@ fn update_contract_registry_owners(
     };
 
     // Check if signer is an owner or an admin
-    can_update_contract_registry(contract_registry.clone(), signer, state)?;
+    can_update_contract_registry(contract_registry.clone(), signer, state, admin_permissions)?;
 
     let contract_registry = contract_registry
         .into_builder()
@@ -641,6 +637,7 @@ fn create_namespace_registry(
     payload: CreateNamespaceRegistryAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let namespace = payload.namespace();
 
@@ -667,33 +664,11 @@ fn create_namespace_registry(
         }
     }
 
-    let setting = match state.get_admin_setting() {
-        Ok(Some(setting)) => setting,
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Admins not set and only admins can create a namespace registry: {}",
-                signer,
-            )));
-        }
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Unable to check state: {}",
-                err,
-            )));
-        }
-    };
-
-    for entry in setting.get_entries() {
-        if entry.key == ADMINISTRATORS_SETTING_KEY {
-            let values = entry.value.split(',');
-            let value_vec: Vec<&str> = values.collect();
-            if !value_vec.contains(&signer) {
-                return Err(ApplyError::InvalidTransaction(format!(
-                    "Only admins can create a namespace registry: {}",
-                    signer,
-                )));
-            }
-        }
+    if !admin_permissions.is_admin(signer, state)? {
+        return Err(ApplyError::InvalidTransaction(format!(
+            "Only admins can create a namespace registry: {}",
+            signer,
+        )));
     }
 
     let namespace_registry = NamespaceRegistryBuilder::new()
@@ -711,6 +686,7 @@ fn delete_namespace_registry(
     payload: DeleteNamespaceRegistryAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let namespace = payload.namespace();
 
@@ -729,7 +705,7 @@ fn delete_namespace_registry(
             )));
         }
     };
-    can_update_namespace_registry(namespace_registry.clone(), signer, state)?;
+    can_update_namespace_registry(namespace_registry.clone(), signer, state, admin_permissions)?;
 
     if !namespace_registry.permissions().is_empty() {
         return Err(ApplyError::InvalidTransaction(format!(
@@ -744,6 +720,7 @@ fn update_namespace_registry_owners(
     payload: UpdateNamespaceRegistryOwnersAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let namespace = payload.namespace();
 
@@ -764,7 +741,7 @@ fn update_namespace_registry_owners(
     };
 
     // Check if signer is an owner or an admin
-    can_update_namespace_registry(namespace_registry.clone(), signer, state)?;
+    can_update_namespace_registry(namespace_registry.clone(), signer, state, admin_permissions)?;
     let namespace_registry = namespace_registry
         .into_builder()
         .with_owners(payload.owners().to_vec())
@@ -780,6 +757,7 @@ fn create_namespace_registry_permission(
     payload: CreateNamespaceRegistryPermissionAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let namespace = payload.namespace();
     let contract_name = payload.contract_name();
@@ -799,7 +777,7 @@ fn create_namespace_registry_permission(
         }
     };
     // Check if signer is an owner or an admin
-    can_update_namespace_registry(namespace_registry.clone(), signer, state)?;
+    can_update_namespace_registry(namespace_registry.clone(), signer, state, admin_permissions)?;
 
     let new_permission = PermissionBuilder::new()
         .with_contract_name(contract_name.into())
@@ -843,6 +821,7 @@ fn delete_namespace_registry_permission(
     payload: DeleteNamespaceRegistryPermissionAction,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     let namespace = payload.namespace();
     let contract_name = payload.contract_name();
@@ -863,7 +842,7 @@ fn delete_namespace_registry_permission(
         }
     };
     // Check if signer is an owner or an admin
-    can_update_namespace_registry(namespace_registry.clone(), signer, state)?;
+    can_update_namespace_registry(namespace_registry.clone(), signer, state, admin_permissions)?;
 
     // remove old permission for contract
     let mut permissions = namespace_registry.permissions().to_vec();
@@ -897,201 +876,26 @@ fn delete_namespace_registry_permission(
     state.set_namespace_registry(namespace, namespace_registry)
 }
 
-pub(crate) fn is_admin(
-    signer: &str,
-    org_id: &str,
-    state: &mut SabreState,
-) -> Result<(), ApplyError> {
-    let admin = match state.get_agent(signer) {
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Signer is not an agent: {}",
-                signer,
-            )));
-        }
-        Ok(Some(admin)) => admin,
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Failed to retrieve state: {}",
-                err,
-            )));
-        }
-    };
-
-    if admin.org_id() != org_id {
-        return Err(ApplyError::InvalidTransaction(format!(
-            "Signer is not associated with the organization: {}",
-            signer,
-        )));
-    }
-    if !admin.roles().contains(&"admin".to_string()) {
-        return Err(ApplyError::InvalidTransaction(format!(
-            "Signer is not an admin: {}",
-            signer,
-        )));
-    };
-
-    if !admin.active() {
-        return Err(ApplyError::InvalidTransaction(format!(
-            "Admin is not currently an active agent: {}",
-            signer,
-        )));
-    }
-    Ok(())
-}
-
-fn create_smart_permission(
-    payload: CreateSmartPermissionAction,
-    signer: &str,
-    state: &mut SabreState,
-) -> Result<(), ApplyError> {
-    // verify the signer of the transaction is authorized to create smart permissions
-    is_admin(signer, payload.org_id(), state)?;
-
-    // Check if the smart permissions already exists
-    match state.get_smart_permission(payload.org_id(), payload.name()) {
-        Ok(None) => (),
-        Ok(Some(_)) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Smart Permission already exists: {} ",
-                payload.name(),
-            )));
-        }
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Failed to retrieve state: {}",
-                err,
-            )));
-        }
-    };
-
-    // Check that organizations exists
-    match state.get_organization(payload.org_id()) {
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Organization does not exist exists: {}",
-                payload.org_id(),
-            )));
-        }
-        Ok(Some(_)) => (),
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Failed to retrieve state: {}",
-                err,
-            )));
-        }
-    };
-
-    let smart_permission = SmartPermissionBuilder::new()
-        .with_name(payload.name().to_string())
-        .with_org_id(payload.org_id().to_string())
-        .with_function(payload.function().to_vec())
-        .build()
-        .map_err(|_| {
-            ApplyError::InvalidTransaction(String::from("Cannot build smart permission"))
-        })?;
-
-    state.set_smart_permission(payload.org_id(), payload.name(), smart_permission)
-}
-
-fn update_smart_permission(
-    payload: UpdateSmartPermissionAction,
-    signer: &str,
-    state: &mut SabreState,
-) -> Result<(), ApplyError> {
-    // verify the signer of the transaction is authorized to update smart permissions
-    is_admin(signer, payload.org_id(), state)?;
-
-    // verify that the smart permission exists
-    let smart_permission = match state.get_smart_permission(payload.org_id(), payload.name()) {
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Smart Permission does not exist: {} ",
-                payload.name(),
-            )));
-        }
-        Ok(Some(smart_permission)) => smart_permission,
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Failed to retrieve state: {}",
-                err,
-            )));
-        }
-    };
-
-    let smart_permission = smart_permission
-        .into_builder()
-        .with_function(payload.function().to_vec())
-        .build()
-        .map_err(|_| {
-            ApplyError::InvalidTransaction(String::from("Cannot build smart permission"))
-        })?;
-    state.set_smart_permission(payload.org_id(), payload.name(), smart_permission)
-}
-
-fn delete_smart_permission(
-    payload: DeleteSmartPermissionAction,
-    signer: &str,
-    state: &mut SabreState,
-) -> Result<(), ApplyError> {
-    // verify the signer of the transaction is authorized to delete smart permissions
-    is_admin(signer, payload.org_id(), state)?;
-
-    // verify that the smart permission exists
-    match state.get_smart_permission(payload.org_id(), payload.name()) {
-        Ok(None) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Smart Permission does not exists: {} ",
-                payload.name(),
-            )));
-        }
-        Ok(Some(_)) => (),
-        Err(err) => {
-            return Err(ApplyError::InvalidTransaction(format!(
-                "Failed to retrieve state: {}",
-                err,
-            )));
-        }
-    };
-
-    state.delete_smart_permission(payload.org_id(), payload.name())
-}
-
 // helper function to check if the signer is allowed to update a namespace_registry
 fn can_update_namespace_registry(
     namespace_registry: NamespaceRegistry,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     if !namespace_registry.owners().contains(&signer.into()) {
-        let setting = match state.get_admin_setting() {
-            Ok(Some(setting)) => setting,
-            Ok(None) => {
+        match admin_permissions.is_admin(signer, state) {
+            Ok(true) => (),
+            Ok(false) => {
                 return Err(ApplyError::InvalidTransaction(format!(
-                    "Owners or admins not set. Only owners or admins can update or delete a namespace registry: {}",
+                    "Only owners or admins can update or delete a namespace registry: {}",
                     signer,
                 )));
             }
             Err(err) => {
-                return Err(ApplyError::InvalidTransaction(format!(
-                    "Unable to check state: {}",
-                    err,
-                )));
+                return Err(err);
             }
         };
-
-        for entry in setting.get_entries() {
-            if entry.key == ADMINISTRATORS_SETTING_KEY {
-                let values = entry.value.split(',');
-                let value_vec: Vec<&str> = values.collect();
-                if !value_vec.contains(&signer) {
-                    return Err(ApplyError::InvalidTransaction(format!(
-                        "Only owners or admins can update or delete a namespace registry: {}",
-                        signer,
-                    )));
-                }
-            }
-        }
     }
     Ok(())
 }
@@ -1101,36 +905,21 @@ fn can_update_contract_registry(
     contract_registry: ContractRegistry,
     signer: &str,
     state: &mut SabreState,
+    admin_permissions: &dyn AdminPermission,
 ) -> Result<(), ApplyError> {
     if !contract_registry.owners().contains(&signer.into()) {
-        let setting = match state.get_admin_setting() {
-            Ok(Some(setting)) => setting,
-            Ok(None) => {
+        match admin_permissions.is_admin(signer, state) {
+            Ok(true) => (),
+            Ok(false) => {
                 return Err(ApplyError::InvalidTransaction(format!(
                     "Only owners or admins can update or delete a contract registry: {}",
                     signer,
                 )));
             }
             Err(err) => {
-                return Err(ApplyError::InvalidTransaction(format!(
-                    "Unable to check state: {}",
-                    err,
-                )));
+                return Err(err);
             }
         };
-
-        for entry in setting.get_entries() {
-            if entry.key == ADMINISTRATORS_SETTING_KEY {
-                let values = entry.value.split(',');
-                let value_vec: Vec<&str> = values.collect();
-                if !value_vec.contains(&signer) {
-                    return Err(ApplyError::InvalidTransaction(format!(
-                        "Only owners or admins can update or delete a contract registry: {}",
-                        signer,
-                    )));
-                }
-            }
-        }
     }
     Ok(())
 }
